@@ -280,9 +280,9 @@ public abstract class MetaMethod extends ParameterTypes implements Cloneable {
           String name = getName();
           CachedClass declaringClass = getDeclaringClass();
           if (Modifier.isPrivate(getModifiers()))
-            mopName = new StringBuffer().append("this$").append(declaringClass.getSuperClassDistance()).append("$").append(name).toString();
+            mopName = new StringBuilder("this$").append(declaringClass.getSuperClassDistance()).append("$").append(name).toString();
           else 
-            mopName = new StringBuffer().append("super$").append(declaringClass.getSuperClassDistance()).append("$").append(name).toString();
+            mopName = new StringBuilder("super$").append(declaringClass.getSuperClassDistance()).append("$").append(name).toString();
         }
         return mopName;
     }
