@@ -40,24 +40,29 @@ import org.codehaus.groovy.runtime.InvokerInvocationException;
 import org.codehaus.groovy.runtime.ResourceGroovyMethods;
 import org.codehaus.groovy.runtime.StackTraceUtils;
 
-import java.io.*;
+import java.io.BufferedReader;
+import java.io.File;
+import java.io.FileNotFoundException;
+import java.io.FileReader;
+import java.io.FileWriter;
+import java.io.IOException;
+import java.io.InputStreamReader;
+import java.io.PrintStream;
+import java.io.PrintWriter;
 import java.math.BigInteger;
 import java.net.URI;
 import java.net.URISyntaxException;
 import java.net.URL;
 import java.security.AccessController;
 import java.security.PrivilegedAction;
+import java.util.Enumeration;
 import java.util.Iterator;
 import java.util.List;
+import java.util.Properties;
 import java.util.regex.Pattern;
 
 /**
  * A Command line to execute groovy.
- *
- * @author Jeremy Rayner
- * @author Yuri Schimke
- * @author Roshan Dawrani
- * @version $Revision$
  */
 public class GroovyMain {
 
@@ -177,107 +182,46 @@ public class GroovyMain {
     @SuppressWarnings("static-access")
     private static synchronized Options buildOptions() {
         Options options = new Options();
-        options.addOption(OptionBuilder.hasArg().withArgName("path").withDescription("Specify where to find the class files - must be first argument").create("classpath"));
-        options.addOption(OptionBuilder.withLongOpt("classpath").hasArg().withArgName("path").withDescription("Aliases for '-classpath'").create("cp"));
-
         options.addOption(
-            OptionBuilder.withLongOpt("define").
-            withDescription("define a system property").
-            hasArg(true).
-            withArgName("name=value").
-            create('D'));
+            OptionBuilder.hasArg().withArgName("path").withDescription("Specify where to find the class files - must be first argument").create("classpath"));
+        options.addOption(
+            OptionBuilder.withLongOpt("classpath").hasArg().withArgName("path").withDescription("Aliases for '-classpath'").create("cp"));
+        options.addOption(
+            OptionBuilder.withLongOpt("define").withDescription("define a system property").hasArgs(2).withValueSeparator().withArgName("name=value").create('D'));
         options.addOption(
             OptionBuilder.withLongOpt("disableopt").
             withDescription("disables one or all optimization elements. " +
                             "optlist can be a comma separated list with the elements: " +
                             "all (disables all optimizations), " +
                             "int (disable any int based optimizations)").
-            hasArg(true).
-            withArgName("optlist").
-            create());
+            hasArg(true).withArgName("optlist").create());
         options.addOption(
-            OptionBuilder.hasArg(false)
-            .withDescription("usage information")
-            .withLongOpt("help")
-            .create('h'));
+            OptionBuilder.hasArg(false).withDescription("usage information").withLongOpt("help").create('h'));
         options.addOption(
-            OptionBuilder.hasArg(false)
-            .withDescription("debug mode will print out full stack traces")
-            .withLongOpt("debug")
-            .create('d'));
+            OptionBuilder.hasArg(false).withDescription("debug mode will print out full stack traces").withLongOpt("debug").create('d'));
         options.addOption(
-            OptionBuilder.hasArg(false)
-            .withDescription("display the Groovy and JVM versions")
-            .withLongOpt("version")
-            .create('v'));
+            OptionBuilder.hasArg(false).withDescription("display the Groovy and JVM versions").withLongOpt("version").create('v'));
         options.addOption(
-            OptionBuilder.withArgName("charset")
-            .hasArg()
-            .withDescription("specify the encoding of the files")
-            .withLongOpt("encoding")
-            .create('c'));
+            OptionBuilder.withArgName("charset").hasArg().withDescription("specify the encoding of the files").withLongOpt("encoding").create('c'));
         options.addOption(
-            OptionBuilder.withArgName("script")
-            .hasArg()
-            .withDescription("specify a command line script")
-            .create('e'));
+            OptionBuilder.withArgName("script").hasArg().withDescription("specify a command line script").create('e'));
         options.addOption(
-            OptionBuilder.withArgName("extension")
-            .hasOptionalArg()
-            .withDescription("modify files in place; create backup if extension is given (e.g. \'.bak\')")
-            .create('i'));
+            OptionBuilder.withArgName("extension").hasOptionalArg().withDescription("modify files in place; create backup if extension is given (e.g. \'.bak\')").create('i'));
         options.addOption(
-            OptionBuilder.hasArg(false)
-            .withDescription("process files line by line using implicit 'line' variable")
-            .create('n'));
+            OptionBuilder.hasArg(false).withDescription("process files line by line using implicit 'line' variable").create('n'));
         options.addOption(
-            OptionBuilder.hasArg(false)
-            .withDescription("process files line by line and print result (see also -n)")
-            .create('p'));
+            OptionBuilder.hasArg(false).withDescription("process files line by line and print result (see also -n)").create('p'));
         options.addOption(
-            OptionBuilder.withArgName("port")
-            .hasOptionalArg()
-            .withDescription("listen on a port and process inbound lines (default: 1960)")
-            .create('l'));
+            OptionBuilder.withArgName("port").hasOptionalArg().withDescription("listen on a port and process inbound lines (default: 1960)").create('l'));
         options.addOption(
-            OptionBuilder.withArgName("splitPattern")
-            .hasOptionalArg()
-            .withDescription("split lines using splitPattern (default '\\s') using implicit 'split' variable")
-            .withLongOpt("autosplit")
-            .create('a'));
+            OptionBuilder.withArgName("splitPattern").hasOptionalArg().withDescription("split lines using splitPattern (default '\\s') using implicit 'split' variable").withLongOpt("autosplit").create('a'));
         options.addOption(
-            OptionBuilder.withLongOpt("indy")
-            .withDescription("enables compilation using invokedynamic")
-            .create());
+            OptionBuilder.withLongOpt("indy").withDescription("enables compilation using invokedynamic").create());
         options.addOption(
-            OptionBuilder.withLongOpt("configscript")
-            .hasArg().withDescription("A script for tweaking the configuration options")
-            .create());
+            OptionBuilder.withLongOpt("configscript").hasArg().withDescription("A script for tweaking the configuration options").create());
         options.addOption(
-                OptionBuilder.withLongOpt("basescript")
-                .hasArg().withArgName("class").withDescription("Base class name for scripts (must derive from Script)")
-                .create('b'));
+            OptionBuilder.withLongOpt("basescript").hasArg().withArgName("class").withDescription("Base class name for scripts (must derive from Script)").create('b'));
         return options;
-
-    }
-
-    private static void setSystemPropertyFrom(final String nameValue) {
-        if(nameValue==null) throw new IllegalArgumentException("argument should not be null");
-
-        String name, value;
-        int i = nameValue.indexOf("=");
-
-        if (i == -1) {
-            name = nameValue;
-            value = Boolean.TRUE.toString();
-        }
-        else {
-            name = nameValue.substring(0, i);
-            value = nameValue.substring(i + 1, nameValue.length());
-        }
-        name = name.trim();
-
-        System.setProperty(name, value);
     }
 
     /**
@@ -290,10 +234,11 @@ public class GroovyMain {
         List args = line.getArgList();
         
         if (line.hasOption('D')) {
-            String[] values = line.getOptionValues('D');
-
-            for (int i=0; i<values.length; i++) {
-                setSystemPropertyFrom(values[i]);
+            Properties optionProperties = line.getOptionProperties("D");
+            Enumeration<String> propertyNames = (Enumeration<String>) optionProperties.propertyNames();
+            while (propertyNames.hasMoreElements()) {
+                String nextName = propertyNames.nextElement();
+                System.setProperty(nextName, optionProperties.getProperty(nextName));
             }
         }
 
@@ -424,8 +369,9 @@ public class GroovyMain {
      * @throws IOException
      * @deprecated
      */
+    @Deprecated
     public String getText(String uriOrFilename) throws IOException {
-        if (uriPattern.matcher(uriOrFilename).matches()) {
+        if (URI_PATTERN.matcher(uriOrFilename).matches()) {
             try {
                 return ResourceGroovyMethods.getText(new URL(uriOrFilename));
             } catch (Exception e) {
@@ -451,7 +397,7 @@ public class GroovyMain {
         if (isScriptFile) {
             // search for the file and if it exists don't try to use URIs ...
             File scriptFile = huntForTheScriptFile(script);
-            if (!scriptFile.exists() && uriPattern.matcher(script).matches()) {
+            if (!scriptFile.exists() && URI_PATTERN.matcher(script).matches()) {
                 return new GroovyCodeSource(new URI(script));
             }
             return new GroovyCodeSource( scriptFile );
@@ -462,7 +408,7 @@ public class GroovyMain {
     // RFC2396
     // scheme        = alpha *( alpha | digit | "+" | "-" | "." )
     // match URIs but not Windows filenames, e.g.: http://cnn.com but not C:\xxx\file.ext
-    private static final Pattern uriPattern = Pattern.compile("\\p{Alpha}[-+.\\p{Alnum}]*:[^\\\\]*");
+    private static final Pattern URI_PATTERN = Pattern.compile("\\p{Alpha}[-+.\\p{Alnum}]*:[^\\\\]*");
 
     /**
      * Search for the script file, doesn't bother if it is named precisely.
@@ -538,8 +484,8 @@ public class GroovyMain {
             try {
                 processReader(s, reader, writer);
             } finally {
-                reader.close();
                 writer.close();
+                reader.close();
             }
 
         } else {
@@ -566,11 +512,12 @@ public class GroovyMain {
 
         if (!editFiles) {
             BufferedReader reader = new BufferedReader(new FileReader(file));
+            PrintWriter writer = new PrintWriter(System.out);
+
             try {
-                PrintWriter writer = new PrintWriter(System.out);
                 processReader(s, reader, writer);
-                writer.flush();
             } finally {
+                writer.close();
                 reader.close();
             }
         } else {
@@ -586,14 +533,12 @@ public class GroovyMain {
                 throw new IOException("unable to rename " + file + " to " + backup);
 
             BufferedReader reader = new BufferedReader(new FileReader(backup));
+            PrintWriter writer = new PrintWriter(new FileWriter(file));
+
             try {
-                PrintWriter writer = new PrintWriter(new FileWriter(file));
-                try {
-                    processReader(s, reader, writer);
-                } finally {
-                    writer.close();
-                }
+                processReader(s, reader, writer);
             } finally {
+                writer.close();
                 reader.close();
             }
         }
