@@ -19,16 +19,19 @@
 package groovy.util;
 
 import groovy.lang.Closure;
-import java.util.Comparator;
 import org.codehaus.groovy.runtime.typehandling.DefaultTypeTransformation;
 
+import java.io.Serializable;
+import java.util.Comparator;
+
 /**
  * A Comparator which uses a closure to compare 2 values being equal
  * 
  * @author <a href="mailto:james@coredevelopers.net">James Strachan</a>
  */
-public class ClosureComparator<T> implements Comparator<T> {
+public class ClosureComparator<T> implements Comparator<T>, Serializable {
 
+    private static final long serialVersionUID = -4593521535656429522L;
     Closure closure;
 
     public ClosureComparator(Closure closure) {
@@ -36,7 +39,7 @@ public class ClosureComparator<T> implements Comparator<T> {
     }
 
     public int compare(T object1, T object2) {
-        Object value = closure.call(new Object[] {object1, object2});
+        Object value = closure.call(object1, object2);
         return DefaultTypeTransformation.intUnbox(value);
     }
 }
