@@ -18,12 +18,12 @@
  */
 package org.codehaus.groovy.antlr.treewalker;
 
-import java.io.PrintStream;
-import java.util.Stack;
-
 import org.codehaus.groovy.antlr.GroovySourceAST;
 import org.codehaus.groovy.antlr.parser.GroovyTokenTypes;
 
+import java.io.PrintStream;
+import java.util.Stack;
+
 /**
  * A visitor that prints a html tags of each node to the supplied PrintStream
  *
@@ -58,7 +58,7 @@ public class NodeAsHTMLPrinter extends VisitorAdapter {
         }
     }
 
-    private String quote(String tokenName) {
+    private static String quote(String tokenName) {
         if (tokenName.length() > 0 && tokenName.charAt(0) != '\'')
             return "'" + tokenName + "'";
         else
@@ -69,7 +69,7 @@ public class NodeAsHTMLPrinter extends VisitorAdapter {
         out.println("</pre></body></html>");
     }
 
-    private String colour(GroovySourceAST t) {
+    private static String colour(GroovySourceAST t) {
         String black = "000000";
         String blue = "17178B";
         String green = "008000";
