@@ -29,7 +29,6 @@ class X {
 
     static {
         StaticInitTest.failed = true;
-        System.out.println("INIT");
     }
 }
 
@@ -38,12 +37,9 @@ public class StaticInitTest extends TestCase {
     static boolean failed;
 
     public void testInitOrder () throws NoSuchFieldException, IllegalAccessException, ClassNotFoundException {
-        System.out.println("GET FIELD");
         final Field f = new GroovyClassLoader().loadClass("org.codehaus.groovy.runtime.X", false, false, false).getField("field");
-        System.out.println(failed);
         assertTrue(!failed);
         f.getInt(null);
-        System.out.println(failed);
         assertTrue(failed);
     }
 }
