@@ -453,6 +453,9 @@ public class CharScanner {
                 offset++;
                 negative = true;
             }
+            if (offset >= to) {
+                die();
+            }
             if (negative) {
                 num = (digitChars[offset] - '0');
                 if (++offset < to) {
@@ -527,7 +530,9 @@ public class CharScanner {
             offset++;
             negative = true;
         }
-
+        if (offset >= to) {
+            die();
+        }
         c = digitChars[offset];
         num = (c - '0');
         offset++;
@@ -550,7 +555,9 @@ public class CharScanner {
             offset++;
             negative = true;
         }
-
+        if (offset >= to) {
+            die();
+        }
         c = digitChars[offset];
         num = (c - '0');
         offset++;
@@ -573,7 +580,9 @@ public class CharScanner {
             offset++;
             negative = true;
         }
-
+        if (offset >= to) {
+            die();
+        }
         c = digitChars[offset];
         num = (c - '0');
         offset++;
@@ -619,6 +628,9 @@ public class CharScanner {
         if (buffer[index] == '-') {
             index++;
         }
+        if (index >= max) {
+            die();
+        }
 
         boolean foundDot = false;
         for (; index < max; index++) {
@@ -653,23 +665,8 @@ public class CharScanner {
             } else {
                 value = parseLongFromTo(buffer, from, index);
             }
-        } else if (foundDot && simple) {
-            BigDecimal lvalue;
-
-            if (length < powersOf10.length) {
-                if (isInteger(buffer, from, length)) {
-                    lvalue = new BigDecimal(parseIntFromToIgnoreDot(buffer, from, index));
-                } else {
-                    lvalue = new BigDecimal(parseLongFromToIgnoreDot(buffer, from, index));
-                }
-
-                BigDecimal power = new BigDecimal(powersOf10[digitsPastPoint]);
-                value = lvalue.divide(power);
-            } else {
-                value = new BigDecimal(new String(buffer, from, length));
-            }
         } else {
-            value = new BigDecimal(new String(buffer, from, index - from));
+            value = parseBigDecimal(buffer, from, length);
         }
 
         if (size != null) {
@@ -684,7 +681,7 @@ public class CharScanner {
     }
 
     public static BigDecimal parseBigDecimal(char[] buffer, int from, int to) {
-        return new BigDecimal(parseDouble(buffer, from, to));
+        return new BigDecimal(buffer, from, to);
     }
 
     public static float parseFloat(char[] buffer, int from, int to) {
